"""Public package API for qpip."""

from .cli import main
from .core import (
    DEFAULT_MIRROR,
    MIRROR_ALIASES,
    MIRRORS,
    build_pip_command,
    format_mirrors,
    get_default_mirror,
    normalize_mirror_name,
    run_pip_with_mirror,
    should_inject_mirror,
)
from .project_venv import (
    DEFAULT_VENV_DIRNAME,
    build_venv_command,
    detect_shell,
    run_activate_command,
    run_deactivate_command,
    run_venv_command,
)

__all__ = [
    "DEFAULT_MIRROR",
    "DEFAULT_VENV_DIRNAME",
    "MIRROR_ALIASES",
    "MIRRORS",
    "build_pip_command",
    "build_venv_command",
    "detect_shell",
    "format_mirrors",
    "get_default_mirror",
    "main",
    "normalize_mirror_name",
    "run_activate_command",
    "run_deactivate_command",
    "run_pip_with_mirror",
    "run_venv_command",
    "should_inject_mirror",
]

__version__ = "0.1.9"
